/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef amiga_h
#define amiga_h

#include <cairo.h>
#include <stdint.h>

/* #define AMIGA_CAIRO_BITS16 */

#define BPLCON0_ECSENA (1<<0)
#define BPLCON0_ERSY   (1<<1)
#define BPLCON0_LACE   (1<<2)
#define BPLCON0_LPEN   (1<<3)
#define BPLCON0_BPU3   (1<<4)
#define BPLCON0_BYPASS (1<<5)
#define BPLCON0_SHRES  (1<<6)
#define BPLCON0_UHRES  (1<<7)
#define BPLCON0_GAUD   (1<<8)
#define BPLCON0_COLOR  (1<<9)
#define BPLCON0_DPF    (1<<10)
#define BPLCON0_HAM    (1<<11)
#define BPLCON0_BPUx   (7<<12)
#define BPLCON0_HIRES  (1<<15)

#ifdef AMIGA_CAIRO_BITS16
typedef uint16_t amiga_cairo_color_t;
#else
typedef uint32_t amiga_cairo_color_t;
#endif

#define amiga_plane_stride(w) (((w-1)/16+1)*2)

void amiga_decode_palette(const uint16_t *amiga_palette, amiga_cairo_color_t *cairo_palette,int colors);
void amiga_decode_line(const uint8_t **amiga_bitplane, amiga_cairo_color_t *dst, int bplcon0, const amiga_cairo_color_t *palette);
cairo_surface_t *abydos_image_surface_create_from_bitplanes(const uint8_t **bitplane, int width, int height, int depth, const amiga_cairo_color_t *palette, int palette_stepping);
cairo_surface_t *abydos_image_surface_create_from_amiga_bitplanes(const uint8_t **bitplane, int width, int height, int depth, int bplcon0, const amiga_cairo_color_t *, int colors, int palette_stepping);
cairo_surface_t *amiga_decode_image(const uint8_t **amiga_bitplane, const amiga_cairo_color_t *palette);
cairo_surface_t *abydos_image_surface_create_from_amiga_pixels(const uint8_t *pixels, int width, int height, int stride, int bplcon0, amiga_cairo_color_t *palette, int palette_stepping);

#endif /* amiga_h */
