/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef abydos_internal_h
#define abydos_internal_h

#include "pthread.h"

#include "abydos-plugin.h"

typedef struct{
    int refcount;
    char *filename;                     /* Store the filename in order to load it on demand. */
    void *handle;
    pthread_mutex_t lock;
    abydos_plugin_interface_t *vt;
} plugin_t;

const char *_abydos_unalias(const char *mime_type);
plugin_t * _abydos_get_plugin(const char *mime_type);
plugin_t * _abydos_ref_plugin(plugin_t *);
void _abydos_unref_plugin(plugin_t *);

#define PLUGIN_CALL(p,func,arg,...) \
  (p)->vt->func(arg,##__VA_ARGS__)

#define PLUGIN_LOCK_CALL(p,unsafe,func,arg,...) \
  { \
    if(unsafe) pthread_mutex_lock(&(p)->lock); \
    (p)->vt->func(arg,##__VA_ARGS__); \
    if(unsafe) pthread_mutex_unlock(&(p)->lock); \
  }

#define PLUGIN_LOCK_CALL_RET(p,unsafe,func,type,arg,...) \
  ({ \
    type ret; \
    if(unsafe) pthread_mutex_lock(&(p)->lock); \
    ret = (p)->vt->func(arg,##__VA_ARGS__); \
    if(unsafe) pthread_mutex_unlock(&(p)->lock); \
    ret; \
  })

#endif /* abydos_internal_h */
